# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-06-12 16:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("uk_geo_utils", "0005_onsud_ced")]

    operations = [
        migrations.AddField(
            model_name="onspd",
            name="calncv",
            field=models.CharField(blank=True, max_length=9),
        ),
        migrations.AddField(
            model_name="onspd",
            name="stp",
            field=models.CharField(blank=True, max_length=9),
        ),
        migrations.AddField(
            model_name="onspd",
            name="streg",
            field=models.CharField(blank=True, max_length=1),
        ),
    ]
